package com.example.backend.movies.api;

import java.util.List;
import java.util.ArrayList;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.backend.categories.service.CategorieService;
import com.example.backend.core.configurations.Constants;
import com.example.backend.favorites.model.FavoriteEntity;
import com.example.backend.favorites.service.FavoriteService;
import com.example.backend.movies.model.MovieEntity;
import com.example.backend.movies.service.MovieService;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.service.UserService;
import com.example.backend.viewed.model.ViewedEntity;
import com.example.backend.viewed.service.ViewedService;

import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.security.core.Authentication;

import jakarta.validation.Valid;

import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.PostMapping;

@Controller
@RequestMapping(MovieUserController.URL)
public class MovieUserController {

    public static final String URL = "/movies";
    private static final String MOVIE_VIEW = "movies";
    private static final String MOVIE_EDIT_VIEW = "movie-edit";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String MOVIE_ATTRIBUTE = "movie";
    private static final String CATEGORIEID_ATTRIBUTE = "categorieId";
    private static final String MOVEID_ATTRIBUTE = "movieId";

    private final MovieService movieService;
    private final CategorieService categorieService;
    private final UserService userService;
    private final FavoriteService favoriteService;
    private final ViewedService viewedServise;

    private final ModelMapper modelMapper;

    public MovieUserController(MovieService movieService,
            CategorieService categorieService,
            ModelMapper modelMapper,
            UserService userService,
            FavoriteService favoriteService,
            ViewedService viewedServise) {
        this.modelMapper = modelMapper;
        this.categorieService = categorieService;
        this.movieService = movieService;
        this.userService = userService;
        this.favoriteService = favoriteService;
        this.viewedServise = viewedServise;
    }

    private MovieDTO toDto(MovieEntity entity) {
        return modelMapper.map(entity, MovieDTO.class);
    }

    private MovieEntity toEntity(MovieDTO dto) {
        final MovieEntity entity = modelMapper.map(dto, MovieEntity.class);
        entity.setCategorie(categorieService.get(dto.getCategorieId()));
        return entity;
    }

    private List<MovieUserDTO> getListMovieUserDTOs(Integer categorieId, int page) {
        List<MovieDTO> movies = movieService.getAll(categorieId, page, Constants.DEFUALT_PAGE_SIZE)
                .stream()
                .map(this::toDto)
                .toList();
        List<MovieUserDTO> muDTO = new ArrayList<MovieUserDTO>();

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        List<FavoriteEntity> favs = favoriteService.getAll(currentUserId);
        List<ViewedEntity> viewes = viewedServise.getAll(currentUserId);

        for (MovieDTO movieDTO : movies) {
            MovieUserDTO newMovDto = new MovieUserDTO();
            newMovDto.setMovieDTO(movieDTO);

            for (FavoriteEntity favoriteEntity : favs) {
                if ((int) favoriteEntity.getMovie().getId() == (int) movieDTO.getId()) {
                    newMovDto.setFavorite(true);
                    break;
                }
                newMovDto.setFavorite(false);
            }

            for (ViewedEntity viewedEntity : viewes) {
                if ((int) viewedEntity.getMovie().getId() == (int) movieDTO.getId()) {
                    newMovDto.setViewed(true);
                    break;
                }
                newMovDto.setViewed(false);
            }

            muDTO.add(newMovDto);
        }
        return muDTO;
    }

    @GetMapping()
    public String getAll(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        List<MovieUserDTO> muDTO = getListMovieUserDTOs(0, page);
        model.addAttribute("movies", muDTO);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, model);
        model.addAttribute("categories", categorieService.getAll());

        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, 0);
        return MOVIE_VIEW;
    }

    @GetMapping("/categorieId/{id}")
    public String getAll(@PathVariable(name = "id") Integer categorieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        List<MovieUserDTO> muDTO = getListMovieUserDTOs(categorieId, page);

        model.addAttribute("movies", muDTO);
        model.addAttribute("categories", categorieService.getAll());

        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, categorieId);

        return MOVIE_VIEW;
    }

    @PostMapping("/changeFavStatus/{id}")
    public String changeFavStatus(@PathVariable(name = "id") Integer movieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        boolean isFavorite = false;

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        FavoriteEntity fav = favoriteService.findByUserIdAndMovieId(currentUserId, movieId);

        if (fav == null) {
            isFavorite = false;
        } else {
            isFavorite = true;
        }

        if (isFavorite == false) {

            UserEntity user = userService.get(currentUserId);
            MovieEntity movie = movieService.get(movieId);

            FavoriteEntity newFav = new FavoriteEntity(null, user, movie);

            favoriteService.create(newFav);

        } else {

            FavoriteEntity delFav = favoriteService.findByUserIdAndMovieId(currentUserId, movieId);

            favoriteService.delete(delFav.getId());
        }

        model.addAttribute(PAGE_ATTRIBUTE, page);

        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/changeViewStatus/{id}")
    public String changeViewStatus(@PathVariable(name = "id") Integer movieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        boolean isViewed = false;

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        ViewedEntity view = viewedServise.findByUserIdAndMovieId(currentUserId, movieId);

        if (view == null) {
            isViewed = false;
        } else {
            isViewed = true;
        }

        if (isViewed == false) {

            UserEntity user = userService.get(currentUserId);
            MovieEntity movie = movieService.get(movieId);

            ViewedEntity newView = new ViewedEntity(null, user, movie);

            viewedServise.create(newView);

        } else {

            ViewedEntity delView = viewedServise.findByUserIdAndMovieId(currentUserId, movieId);

            viewedServise.delete(delView.getId());
        }

        model.addAttribute(PAGE_ATTRIBUTE, page);

        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/card/{id}")
    public String getCardFilm(@PathVariable(name = "id") Integer movieId,
            Model model) {
        MovieEntity movie = movieService.get(movieId);
        model.addAttribute("movie", movie);
        return "card-film";
    }

    @GetMapping("/countView")
    public Integer countView(@RequestParam(name = "movieId", defaultValue = "0") Integer movieId) {
        return movieService.countView(movieId);
    }
}
